﻿using AutoMapper;
using Correo.Abstractions;
using Correo.Application.Extensions;
using Correo.Domain.Models;
using Correo.PublishedLanguage.Commands;
using Correo.PublishedLanguage.Events;
using MediatR;
using Microsoft.Extensions.Logging;
using Microsoft.Extensions.Options;
using NBB.Messaging.Abstractions;
using System;
using System.Threading;
using System.Threading.Tasks;

namespace Correo.Application.CommandHandlers
{
    public class SendEmailHandler : IRequestHandler<SendEmail>
    {
        private readonly IMessageBusPublisher _messageBusPublisher;
        private readonly IOptions<DefaultSender> _defaultSender;
        private readonly ILogger<SendEmailHandler> _logger;
        private readonly IMapper _mapper;
        private readonly IMailer _mailer;

        public SendEmailHandler(IMessageBusPublisher messageBusPublisher, IOptions<DefaultSender> defaultSender, ILogger<SendEmailHandler> logger, IMapper mapper, IMailer mailer)
        {
            _messageBusPublisher=messageBusPublisher;
            _defaultSender=defaultSender;
            _logger=logger;
            _mapper=mapper;
            _mailer=mailer;
        }

        public async Task<Unit> Handle(SendEmail command, CancellationToken cancellationToken)
        {
            try
            {
                var emailMessage = _mapper.Map<EmailMessage>(command);
                emailMessage.Enrich(_defaultSender.Value);
                emailMessage.Validate();
                await _mailer.SendEmailAsync(emailMessage, cancellationToken);
                await _messageBusPublisher.PublishAsync(new EmailSent(command.Subject), cancellationToken);
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, ex.Message);
                await _messageBusPublisher.PublishAsync(new EmailSentFailed(command.Subject, ex.Message), cancellationToken);
                throw;
            }

            return Unit.Value;
        }
    }
}
